/*
 * Here comes the text of your license
 * Each line should be prefixed with  * 
 */
package gr.ihu.cs.lmous.OOJ.k11.v1.x;

public class Weight { //Κώδικας 11.14

    private double value;
    private WeightMeasurementUnit weightUnit;

    public Weight(double value, String unitDescription, double relationToGr) {
        this(value, new WeightMeasurementUnit(unitDescription, relationToGr));
    }

    public Weight(double value, WeightMeasurementUnit mB) {
        if (value<0) throw new RuntimeException();
        this.value = value;
        this.weightUnit = mB;
    }

    public Weight(Weight w) {
        this(w.value, w.weightUnit);
    }

    public static Weight convert(WeightMeasurementUnit from, 
            WeightMeasurementUnit to) {
        double valueTo = from.getRelationToGr() / to.getRelationToGr();
        return new Weight(valueTo, to);
    }

    public Weight convert(WeightMeasurementUnit wUnit) {
        Weight rVal = convert(weightUnit, wUnit);
        rVal.value *= value;
        return rVal;
    }

    public Weight add(Weight w) {
        //Weight w1 = new Weight(w);
        if (!weightUnit.equals(w.weightUnit)) {
            w = w.convert(weightUnit);
        }
        return new Weight(value + w.value, weightUnit);
    }

    public void setValue(double value) {
        this.value = value;
    }

    public double getValue() {
        return value;
    }

    public WeightMeasurementUnit getWeightUnit() {
        return weightUnit;
    }

    public void setWeightUnit(WeightMeasurementUnit weightUnit) {
        this.weightUnit = weightUnit;
    }

    @Override
    public int hashCode() {
        Weight w = convert(new WeightMeasurementUnit("gr", 1));
        int hash = 7;
        hash = 23 * hash + Double.hashCode(w.value);
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        Weight other = (Weight) obj;
        other = other.convert(this.weightUnit);
        return Double.compare(this.value, other.value) == 0;
    }

    @Override
    public String toString() {
        return value + " " + weightUnit.getDescription();
    }

    public static void main(String[] args) {
        WeightMeasurementUnit kgr = new WeightMeasurementUnit("kgr", 1000);
        WeightMeasurementUnit mgr = new WeightMeasurementUnit("mgr", 1 / 1000d);
        Weight w1 = new Weight(1, kgr);
        Weight w2 = new Weight(100, mgr);
        Weight w3 = w1.add(w2);
        Weight w4 = w2.add(w1);
        System.out.println(w3);
        System.out.println(w4);
        System.out.println(w3.equals(w4));
        System.out.println(w3.hashCode() == w4.hashCode());
    }
}
