/*
 * Here comes the text of your license
 * Each line should be prefixed with  * 
 */
package gr.ihu.cs.lmous.OOJ.k11.v3;

public class Rectangle { //Κώδικας 11.8 

    private double length, width;

    public Rectangle(double length, double width) {
        if (length <= 0 || width <= 0) {
            throw new IllegalArgumentException();
        }
        this.length = length;
        this.width = width;
    }

    public Rectangle(double length) {
        this(length, length);
    }

    public Rectangle() {
        this.length = 2;
        this.width = 2;
    }

    public double area() {
        return length * width;
    }

    public double perimeter() {
        return 2 * (this.length + this.width);
    }

    public double diagonal() {
        return Math.sqrt(Math.pow(this.length, 2) + Math.pow(this.width, 2));
    }

    public double getLength() {
        return this.length;
    }

    public void setLength(double length) {
        if (length <= 0) {
            throw new IllegalArgumentException();
        }
        this.length = length;
    }

    public double getWidth() {
        return this.width;
    }

    public void setWidth(double width) {
        if (width <= 0) {
            throw new IllegalArgumentException();
        }
        this.width = width;
    }

    @Override
    public String toString() {
        return "Rectangle{" + "length=" + length + ", width=" + width + '}';
    }

    @Override
    public int hashCode() {
        return Double.hashCode(length) + Double.hashCode(width);
    }
    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Rectangle other = (Rectangle) obj;
        if (Double.compare(width, other.width) != 0) {
            return false;
        }
        return Double.compare(length, other.length) == 0;
    }

    public boolean aproximateEquals(Rectangle r) {
        if (equals(r)) {
            return true;
        }
        return Math.abs(width - r.width) < 0.001 && Math.abs(length - r.length) < 0.001;
    }

    public static void main(String[] args) {
        Rectangle r1 = new Rectangle(2, 6), r2 = new Rectangle(3, 4);
        System.out.println(r1.toString());
        System.out.println(r1 == r2);
        System.out.println(r1.hashCode() + " " + r2.hashCode());
    }
}
