/**
 * @author Lefteris Moussiades
 * @since 20/09/2015
 */
package gr.ihu.cs.lmous.OOJ.k14_1.v1.x.TicTacToe;

/**
 * Αναπαριστά το ταμπλό της Τρίλιζας
 *
 * @author Lefteris
 */
public class Board {

    private final char[][] board;

    /**
     * Σημειώνει τις άδειες θέσεις στο ταμπλό
     */
    public static final char EMPTYCELL = ' ';

    /**
     * Ο Δημιουργός της κλάσης. Δημιουργεί ένα ταμπλό Τρίλιζας. Όλες οι θέσεις
     * αρχικοποιούνται στο EMPTYCELL
     */
    public Board() {
        board = new char[3][3];
        for (int i = 0; i < 3; i++) {
            for (int j = 0; j < 3; j++) {
                board[i][j] = EMPTYCELL;
            }
        }
    }

    /**
     * Δημιουργεί ένα αντίγραφο του ταμπλό
     *
     * @param source Το ταμπλό αντίγραφο του οποίου θα δημιουργηθεί
     */
    public Board(Board source) {
        board = new char[3][3];
        for (int i = 0; i < 3; i++) {
            System.arraycopy(source.board[i], 0, board[i], 0, 3);
        }
    }

    /**
     * Δημιουργεί αυτόματα μια κίνηση για τον παίκτη p. Καλείται μόνο εφόσον υπάρχει
     * μια μόνο θέση ελεύθερη στο ταμπλό και είναι σειρά του παίκτη p.
     *
     * @param p
     */
    void autoMove(Player p) {
        for (int i = 0; i < 3; i++) {
            for (int j = 0; j < 3; j++) {
                if (board[i][j] == EMPTYCELL) {
                    board[i][j] = p.getId();
                }
            }
        }
    }

    /**
     * Εμφανίζει το ταμπλό
     */
    void displayBoard() {
        System.out.println("  0 1 2");
        for (int i = 0; i < 3; i++) {
            System.out.println("--------");
            System.out.print(i);
            for (int j = 0; j < 3; j++) {
                System.out.print("|" + board[i][j]);
            }
            System.out.println("|");
        }
        System.out.println("--------");
    }

    /**
     * Θέτει την κίνηση mv του παίκτη p στο ταμπλό
     *
     * @param p
     * @param mv
     */
    public void setPlayerMove(Player p, Movement mv) {
        board[mv.getRow()][mv.getClmn()] = p.getId();
    }

    /**
     *
     * @return τον αριθμό των ελεύθερων θέσεων στο ταμπλό
     */
    public int freePositions() {
        int cnt = 0;
        for (int i = 0; i < 3; i++) {
            for (int j = 0; j < 3; j++) {
                if (board[i][j] == EMPTYCELL) {
                    cnt++;
                }
            }
        }
        return cnt;
    }

    private char lineWin(int ln) {
        if ((board[ln][0] != EMPTYCELL)
                && (board[ln][0] == board[ln][1])
                && (board[ln][0] == board[ln][2])) {
            return board[ln][0];
        }
        return EMPTYCELL;
    }

    private char columnWin(int clmn) {
        if ((board[0][clmn] != EMPTYCELL)
                && (board[0][clmn] == board[1][clmn])
                && (board[1][clmn] == board[2][clmn])) {
            return board[0][clmn];
        }
        return EMPTYCELL;
    }

    private char diagonalWin() {
        if ((board[0][0] != EMPTYCELL)
                && (board[0][0] == board[1][1])
                && (board[0][0] == board[2][2])) {
            return board[0][0];
        }
        if ((board[2][0] != EMPTYCELL)
                && (board[2][0] == board[1][1])
                && (board[2][0] == board[0][2])) {
            return board[2][0];
        }
        return EMPTYCELL;
    }

    /**
     *
     * @return την ταυτότητα του νικητή. 
     * Αν δεν υπάρχει νικητής επιστρέφει EMPTYCELL;
     */
    public char win() {
        char rslt;
        for (int i = 0; i < 3; i++) {
            if ((rslt = lineWin(i)) != EMPTYCELL) {
                return rslt;
            }
            if ((rslt = columnWin(i)) != EMPTYCELL) {
                return rslt;
            }
        }
        return diagonalWin();
    }

    /**
     *
     * @return ένα πίνακα που περιέχει τις ελεύθερες θέσεις του ταμπλό
     */
    public Movement[] possibleMoves() {
        Movement[] rVal;
        int cnt = 0;
        for (int i = 0; i < 3; i++) {
            for (int j = 0; j < 3; j++) {
                if (board[i][j] == EMPTYCELL) {
                    cnt++;
                }
            }
        }
        if (cnt == 0) {
            return null;
        }
        rVal = new Movement[cnt];

        cnt = 0;
        for (int i = 0; i < 3; i++) {
            for (int j = 0; j < 3; j++) {
                if (board[i][j] == EMPTYCELL) {
                    rVal[cnt++] = new Movement(i, j);
                }
            }
        }
        return rVal;
    }

}
