/*
 * Here comes the text of your license
 * Each line should be prefixed with  * 
 */
package gr.ihu.cs.lmous.OOJ.k14_1.v1.x1;

import java.util.ArrayList;
import java.util.Iterator;

public class TableOfChemichalElements {// Κώδικας 14.20

    private final ArrayList<ChemicalElement> store = new ArrayList<>();

    public class GroupIterator implements Iterator {

        private final int groupId;
        private int current = 0;

        public GroupIterator(int groupId) {
            this.groupId = groupId;
        }

        @Override
        public boolean hasNext() {

            while (current < store.size()) {
                ChemicalElement cE = store.get(current);
                if (cE.getGroup() == groupId) {
                    return true;
                }
                current++;
            }
            return false;
        }

        @Override
        public Object next() {
            return store.get(current++);
        }

        @Override
        public void remove() {
            store.remove(--current);
        }
    }

    public Iterator groupIterator(int groupId) {
        return new GroupIterator(groupId);
    }

    public boolean add(ChemicalElement cE) {
        if (store.contains(cE)) {
            return false;
        }
        return store.add(cE);
    }

    public TableOfChemichalElements getGroup(int groupId) {
        TableOfChemichalElements rVal = new TableOfChemichalElements();
        for (ChemicalElement cE : store) {
            if (cE.getGroup() == groupId) {
                rVal.add(cE);
            }
        }
        return rVal;
    }

    public TableOfChemichalElements getPeriod(int periodId) {
        TableOfChemichalElements rVal = new TableOfChemichalElements();
        for (ChemicalElement cE : store) {
            if (cE.getPeriod() == periodId) {
                rVal.add(cE);
            }
        }
        return rVal;
    }

    public TableOfChemichalElements getSection(char sectionId) {
        TableOfChemichalElements rVal = new TableOfChemichalElements();
        for (ChemicalElement cE : store) {
            if (cE.getSection() == sectionId) {
                rVal.add(cE);
            }
        }
        return rVal;
    }

    public boolean isEmpty() {
        return store.isEmpty();
    }

    @Override
    public String toString() {
        StringBuilder t = new StringBuilder();
        for (ChemicalElement cE : store) {
            t.append(cE).append("\n");
        }
        return t.toString();
    }

    public static TableOfChemichalElements setUp() {
        ChemicalElement h = new ChemicalElement(1, 1, 's', 1, "H");
        //int group, int period, char sector, int atomicNumber, String name
        ChemicalElement li = new ChemicalElement(1, 2, 's', 3, "Li");
        ChemicalElement na = new ChemicalElement(1, 3, 's', 11, "Na");
        ChemicalElement be = new ChemicalElement(2, 2, 's', 4, "Be");
        ChemicalElement mg = new ChemicalElement(2, 3, 's', 12, "Mg");
        ChemicalElement sc = new ChemicalElement(3, 4, 'd', 21, "Sc");
        ChemicalElement y = new ChemicalElement(3, 5, 'd', 39, "Y");
        ChemicalElement he = new ChemicalElement(18, 1, 'p', 2, "He");
        ChemicalElement la = new ChemicalElement(3, 6, 'f', 57, "La");
        TableOfChemichalElements gen = new TableOfChemichalElements();
        gen.add(h);
        gen.add(li);
        gen.add(na);
        gen.add(be);
        gen.add(mg);
        gen.add(sc);
        gen.add(y);
        gen.add(he);
        gen.add(la);
        return gen;
    }

    public static void main(String[] args) {

        TableOfChemichalElements gen = setUp();
        System.out.println(gen);
        TableOfChemichalElements group1 = gen.getGroup(1);
        System.out.println(group1);
        System.out.println(gen.getPeriod(1));
        System.out.println(gen.getSection('p'));

        System.out.println("----------------------------");
        Iterator it = gen.groupIterator(1);
        while (it.hasNext()) {
            ChemicalElement cE = (ChemicalElement) it.next();
            if (cE.getPeriod() == 2) {
                it.remove();
            }
            System.out.println(cE);
        }
        System.out.println("----------------------------");
        System.out.println(gen);
    }
}
