/*
 * Here comes the text of your license
 * Each line should be prefixed with  * 
 */
package gr.ihu.cs.lmous.OOJ.k17.x;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

public class MyHashMap<K, V> { //Κώδικας 17.13

    public class Entry<K, V> {

        private final K key;
        private V value;

        public Entry(K key, V value) {
            this.key = key;
            this.value = value;
        }

        public K getKey() {
            return key;
        }

        public V getValue() {
            return value;
        }

        public void setValue(V value) {
            this.value = value;
        }

        @Override
        public int hashCode() {
            int hash = 7;
            hash = 97 * hash + Objects.hashCode(this.key);
            return hash;
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            final Entry<K, V> other = (Entry<K, V>) obj;
            return Objects.equals(this.key, other.key);
        }
    }

    private int find(K key, List<Entry<K, V>> list) {
        for (int i = 0; i < list.size(); i++) {

            Entry<K, V> entry = list.get(i);
            if (entry.key.equals(key)) {
                return i;
            }
        }
        return -1;
    }

    private int size = 0;
    private static final double LOADFACTOR = 0.75;
    private int noOfLists = 0;
    private final int INITIALCAPACITY = 3;
    private int capacity = INITIALCAPACITY;
    private ArrayList<Entry<K,V>>[] store = new ArrayList[INITIALCAPACITY];

    public void clear() {
        store = new ArrayList[INITIALCAPACITY];
        size = 0;
        capacity = INITIALCAPACITY;
        noOfLists = 0;
    }

    private void rehash() {
        int newCapacity = 2 * capacity;
        int newEntries = 0;
        ArrayList<Entry<K,V>>[] local = new ArrayList[newCapacity];
        for (ArrayList<Entry<K,V>> list : store) {
            
            if (list != null) {
                for (Entry<K,V> entry : list) {
                                 int hashCode = entry.key.hashCode();
                    int idx = hashCode % newCapacity;
                    if (local[idx] == null) {
                        local[idx] = new ArrayList<Entry<K, V>>();
                        newEntries++;
                    }
                    local[idx].add(entry);
                }
            }
        }
        store = local;
        noOfLists = newEntries;
        capacity = newCapacity;
    }

    public V put(K key, V value) {
        if (noOfLists > LOADFACTOR * capacity) {
            rehash();
        }
        int idx = key.hashCode() % store.length;
        if (store[idx] == null) {
            store[idx] = new ArrayList<Entry<K, V>>();
            noOfLists++;
        }
        Entry<K, V> entry = new Entry<>(key, value);
        int location = store[idx].indexOf(entry);
        Entry<K, V> oldEntry;
        if (location < 0) {
            store[idx].add(entry);
            size++;
            return null;
        } else {
            oldEntry = store[idx].get(location);
            store[idx].set(location, entry);
        }
        return oldEntry.getValue();
    }

    public V get(K key) {
        int idx = key.hashCode() % store.length;

        int location = find(key, store[idx]);
        if (location < 0) {
            return null;
        }
        return store[idx].get(location).getValue();
    }

    public V remove(K key) {
        int idx = key.hashCode() % store.length;
        int location = find(key, store[idx]);
        if (location < 0) {
            return null;
        }
        size--;
        V rVal = store[idx].remove(location).getValue();
        if (store[idx].isEmpty()) {
            noOfLists--;
            store[idx] = null;
        }
        return rVal;
    }

    public boolean isEmpty() {
        return size==0;
    }
    
    public static void main(String[] args) {
        MyHashMap<String, String> hM = new MyHashMap<>();
        hM.put("key1", "value1");
        hM.put("key2", "value2");
        hM.put("key3", "value3");
        hM.put("key4", "value4");
        hM.put("key5", "value5");
        hM.put("key6", "value6");
        hM.put("key7", "value7");
        hM.put("key8", "value8");
        System.out.println(hM.get("key1"));
        System.out.println(hM.get("key2"));
        System.out.println(hM.get("key3"));
        System.out.println(hM.get("key4"));
        System.out.println(hM.get("key5"));
        System.out.println(hM.get("key6"));
        System.out.println(hM.get("key7"));
        System.out.println(hM.get("key8"));

        System.out.println(hM.remove("key1"));
        System.out.println(hM.isEmpty());
        System.out.println(hM.remove("key2"));
        System.out.println(hM.isEmpty());
        System.out.println(hM.remove("key3"));
        System.out.println(hM.isEmpty());
        System.out.println(hM.remove("key4"));
        System.out.println(hM.isEmpty());
        System.out.println(hM.remove("key5"));
        System.out.println(hM.isEmpty());
        System.out.println(hM.remove("key6"));
        System.out.println(hM.isEmpty());
        System.out.println(hM.remove("key7"));
        System.out.println(hM.isEmpty());
        System.out.println(hM.remove("key8"));
        System.out.println(hM.isEmpty());
    }

}
