package gr.ihu.cs.lmous.OOJ.k8.v2;
//Κώδικας 8.6

import java.util.Random;

/**
 * Η κλάση προσομοιώνει το ακόλουθο παιχνίδι ζαριών. Τρεις φίλοι, ο John, ο Jim
 * και ο Jack παίζουν ένα παιχνίδι ζαριών. Σύμφωνα με τους κανόνες του
 * παιχνιδιού, κατά την διάρκεια μιας παρτίδας, οι παίκτες ρίχνουν εναλλάξ το
 * ζάρι 10 φορές και μετράνε πόσες φορές έφερε ο καθένας τους άρτια ζαριά.
 * Νικητής είναι εκείνος που έφερε τις περισσότερες άρτιες ζαριές. Αν όμως 2 ή
 * περισσότεροι παίκτες έχουν ισοβαθμίσει, η παρτίδα επαναλαμβάνεται για όλους.
 * Η επανάληψη συνεχίζεται μέχρις ότου να επικρατήσει ένας μοναδικός νικητής. Οι
 * συγκεκριμένοι παίκτες όμως έχουν τις ιδιομορφίες τους. Ο John είναι ένας
 * τίμιος παίκτης με την έννοια πως κάθε φορά που πετάει το ζάρι έχει την ίδια
 * πιθανότητα να φέρει οποιοδήποτε αποτέλεσμα από 1 έως 6. Από την άλλη μεριά, ο
 * Jim είναι τυχερός καθώς έχει αποδειχτεί πως όταν πετάει το ζάρι έχει 15%
 * πιθανότητα για κάθε ένα από τα αποτελέσματα 1, 3 και 5, 18% για κάθε ένα από
 * τα αποτελέσματα 2 ή 4 και 19% για να φέρει 6. Επομένως, ο Jim φέρνει άρτια
 * ζαριά με πιθανότητα 55% και περιττή ζαριά με πιθανότητα 45%. Αντίθετα, ο Jack
 * είναι άτυχος καθώς φέρνει 2, 4, 6 με πιθανότητα 15% για κάθε ένα από αυτά τα
 * αποτελέσματα, ενώ 1 και 3 φέρνει με πιθανότητα 18% και 5 φέρνει με πιθανότητα
 * 19%. Επομένως, ο Jack φέρνει άρτια ζαριά με πιθανότητα 45% και περιττή ζαριά
 * με πιθανότητα 55%.
 *
 * @author Lefteris Moussiades
 * @version "%I%, %G%"
 *
 */
public class ADiceGame {

    /**
     * Χρησιμοποιείται απο τον έλεγχο ισότητας πραγματικών
     *
     * @see approximateEquals
     */
    static final double EPSILON = 0.00001;
    private static final Random gen = new Random();
    static final double[] fair = {1 / 6d, 1 / 6d, 1 / 6d, 1 / 6d, 1 / 6d, 1 / 6d};
    static final double[] lucky = {15d / 100, 18d / 100, 15d / 100, 18d / 100, 15d / 100, 19d / 100};
    static final double[] unlucky = {18d / 100, 15d / 100, 18d / 100, 15d / 100, 19d / 100, 15d / 100};
    private static final int JOHN = 0;
    private static final int JIM = 1;
    private static final int JACK = 2;

    /**
     * Συγκκρίνει προσεγγιστικά δύο πραγματικούς τύπου double για έλεγχο
     * ισότητας
     *
     * @param d1
     * @param d2
     * @return Επιστρέφει true αν η απόλυτη τιμή της διαφορας d1-d2 είναι
     * μικρότερη απο την τιμή της EPSILON
     * @see EPSILON
     */
    public static boolean approximateEquals(double d1, double d2) {
        return Math.abs(d1 - d2) < EPSILON;
    }

    /**
     * Υπολογίζει το άθροισμα των στοιχείων της παραμέτρου array
     *
     * @param array
     * @return Το άθροισμα των στοιχείων της array
     */
    public static double sum(double[] array) {
        double sum = 0;
        for (double d : array) {
            sum += d;
        }
        return sum;
    }

    /**
     * Προσομοιώνει την ρίψη ζαριού
     *
     * @param distribution Πίνακας πραγματικών. Κατά την κλήση της μεθόδου, η
     * distribution πρέπει να περιλαμβάνει 6 στοιχεία με άθροισμα ίσο με 1.
     * @return Έναν αριθμό απο 1 έως 6 ανάλογα με την πιθανότητα που δίνεται για
     * κάθε τιμή μέσω της παραμέτρου distribution
     * @throws RuntimeException
     */
    public static int roll(double[] distribution) {
        if (distribution.length != 6 || !approximateEquals(sum(distribution), 1d)) {
            throw new RuntimeException();
        }
        double value = gen.nextDouble();
        double sum = 0;
        int rslt = 1;
        for (double percentage : distribution) {
            sum += percentage;
            if (value <= sum) {
                return rslt;
            }
            rslt++;
        }
        return 0;
    }

    /**
     * Εντοπίζει την μέγιστη τιμή του πίνακα array
     *
     * @param array Ο πίνακας του οποίου το μέγιστο στοιχείο θέλουμε να βρούμε
     * @return Επιστρέφει την θέση της πρώτης απο τις μέγιστες τιμές του πίνακα
     * array
     */
    public static int max(int[] array) {
        int idx = 0;
        int max = array[0];
        for (int i = 1; i < array.length; i++) {
            if (array[i] > max) {
                max = array[i];
                idx = i;
            }
        }
        return idx;
    }

    /**
     * Υπολογίζει πόσες φορές υπάρχει μια τιμή σε ένα πίνακα
     *
     * @param array Ο πίνακας στον οποίο θα μετρήσουμε πόσες φορές υπάρχει η
     * value
     * @param value Η τιμή που θα μετρήσουμε πόσες φορές υπάρχει στον array
     * @return Επιστρέφει το πλήθος των στοιχείων με τιμή ίση με value στον
     * πίνακα array
     */
    public static int cntOccurances(int[] array, int value) {
        int cnt = 0;
        for (int i = 0; i < array.length; i++) {
            if (value == array[i]) {
                cnt++;
            }
        }
        return cnt;
    }

    private static boolean withdraw(int[] score) {
        return cntOccurances(score, score[max(score)]) > 1;
    }

    /*
    Η aLot  λαμβάνει ως παράμετρο τον αριθμό των ρίψεων ανα παρτίδα και παίκτη. 
    Δηλώνει τον πίνακα score όπου αποθηκεύονται οι ζαριές με άρτιο αποτέλεσμα για κάθε παίκτη κατά την διάρκεια της παρτίδας.
    Προκειμένου να αντιστοιχίζεται κάθε παίκτης ευκολότερα, με περισσότερη ασφάλεια και να είναι το πρόγραμμα ευανάγνωστο, 
    έχουν δηλωθεί οι σταθερές JOHN, JIM και JACK ως 0, 1 και 2. Έτσι γίνεται μια αντιστοιχία μεταξύ των θέσεων στον πίνακα των σκορ 
    με τα ονόματα των παικτών. 
    Μέσα στον βρόγχο for της aLot κάθε παίκτης ρίχνει την ζαριά του με την βοήθεια της roll και αν το αποτέλεσμα είναι άρτιο ενημερώνεται 
    το σκορ του.
     */
    private static int[] aLot(int noOfRolls) {
        int[] score = new int[3];
        for (int i = 0; i < noOfRolls; i++) {
            int rslt = roll(fair);
            if (rslt % 2 == 0) {
                score[JOHN]++;
            }

            rslt = roll(lucky);
            if (rslt % 2 == 0) {
                score[JIM]++;
            }

            rslt = roll(unlucky);
            if (rslt % 2 == 0) {
                score[JACK]++;
            }
        }
        return score;
    }

    /*
    Η game υλοποιεί ένα παιχνίδι με συγκεκριμένα χαρακτηριστικά παρτίδας. 
    Έτσι δηλώνει ως τοπική σταθερά τον αριθμό ρίψεων κάθε παρτίδας. 
    Στην συνέχεια, σύμφωνα με τον σχεδιασμό καλεί την aLot μέσα σε επαναληπτική 
    διαδικασία που ελέγχεται από την withdraw. 
    Σε κάθε κλήση, η aLot επιστρέφει έναν πίνακα με τα σκορ των παικτών. 
    Τα σκορ ελέγχει η withdraw. Αν η withdraw δεν ανιχνεύσει ισοπαλία τερματίζεται 
    ο βρόχος do και επιστρέφεται η θέση του πίνακα 
    των σκορ με την μεγαλύτερη τιμή. Εκεί βρίσκεται ο νικητής.  
     */
    private static int game() {
        final int NOOFROLLS = 10;
        int[] score;

        do {
            score = aLot(NOOFROLLS);
        } while (withdraw(score));
        return max(score);
    }

    private static String winnerName(int winnerId) {

        switch (winnerId) {
            default:
                throw new RuntimeException();
            case JOHN:
                return "John";
            case JIM:
                return "Jim";
            case JACK:
                return "Jack";
        }
    }

    public static void main(String[] args) {
        int winnerId = game();
        System.out.println("winner is " + winnerName(winnerId));

    }
}

